@@ -11,13 +11,8 @@
 
 namespace Dune
 {
-
-  namespace Alberta
-  {
-    IndexStack *currentIndexStack = 0;
-  }
-
-
+  template< int dim, int dimworld >
+  Alberta::IndexStack* AlbertaGridHierarchicIndexSet<dim,dimworld>::currentIndexStack = nullptr;
 
   // Implementation of AlbertaGridHierarchicIndexSet::CreateEntityNumbers
   // --------------------------------------------------------------------
@@ -142,7 +137,7 @@ namespace Dune
   template< int dim, int dimworld >
   void AlbertaGridHierarchicIndexSet< dim, dimworld >::create ()
   {
-    Hybrid::forEach( Std::make_index_sequence< dimension+1 >{},
+    Hybrid::forEach( std::make_index_sequence< dimension+1 >{},
       [ & ]( auto i ){ CreateEntityNumbers< i >::apply( dofNumbering_, *this ); } );
   }
 
@@ -150,7 +145,7 @@ namespace Dune
   template< int dim, int dimworld >
   void AlbertaGridHierarchicIndexSet< dim, dimworld >::read ( const std::string &filename )
   {
-    Hybrid::forEach( Std::make_index_sequence< dimension+1 >{},
+    Hybrid::forEach( std::make_index_sequence< dimension+1 >{},
       [ & ]( auto i ){ CreateEntityNumbers< i >::apply( filename, dofNumbering_.mesh(), *this ); } );
   }
 
