@@ -61,8 +61,8 @@ public:
      @param[in]  xi     point in local coordinates of the reference element of e
      \return            value of the component
    */
-  virtual double evaluate (int comp, const Entity& e,
-                           const Dune::FieldVector<DT,n>& xi) const
+  virtual double evaluate (int comp, [[maybe_unused]] const Entity& e,
+                           [[maybe_unused]] const Dune::FieldVector<DT,n>& xi) const
   {
     return comp*0.1;
   }
@@ -94,7 +94,7 @@ struct Acc
 };
 
 template< class GridView >
-int doWrite( Dune::VTKChecker& vtkChecker, const std::string& gridViewName, const GridView &gridView, Dune :: VTK :: DataMode dm )
+int doWrite( Dune::VTKChecker& vtkChecker, const std::string& gridViewName, const GridView &gridView, Dune :: VTK :: DataMode dm, const std::string& path = "./" )
 {
   enum { dim = GridView :: dimension };
 
@@ -137,6 +137,8 @@ int doWrite( Dune::VTKChecker& vtkChecker, const std::string& gridViewName, cons
   int result = 0;
   std::string name;
   std::ostringstream prefix;
+  prefix << path << "/";
+
   prefix << "vtktest-" << dim << "D-" << gridViewName << "-" << VTKDataMode(dm);
   int rank = gridView.comm().rank();
 
@@ -182,6 +184,11 @@ int vtkCheck(Dune::VTKChecker& vtkChecker, const std::array<int, dim>& elements,
   acc(result, doWrite( vtkChecker, "finelevelview", g.levelGridView( g.maxLevel() ),
                        Dune::VTK::nonconforming ));
 
+  // write with filename including a path which is the current directory
+  // this is for testing the parallel version of 'write'
+  acc(result, doWrite( vtkChecker, "leafview", g.leafGridView(), Dune::VTK::conforming, "../test" ));
+  acc(result, doWrite( vtkChecker, "leafview", g.leafGridView(), Dune::VTK::nonconforming, "../test" ));
+
   return result;
 }
 
