@@ -68,7 +68,7 @@ insertBoundarySegment(const std::vector<unsigned int>& vertices)
 
 void Dune::GridFactory<Dune::OneDGrid>::
 insertBoundarySegment(const std::vector<unsigned int>& vertices,
-                      const std::shared_ptr<BoundarySegment<1> > &boundarySegment)
+                      const std::shared_ptr<BoundarySegment<1> > & /* boundarySegment */)
 {
   insertBoundarySegment(vertices);
 }
@@ -100,7 +100,7 @@ insertionIndex(const typename OneDGrid::LeafIntersection& intersection) const
   return insertionIdx;
 }
 
-ToUniquePtr<OneDGrid> Dune::GridFactory<Dune::OneDGrid>::
+std::unique_ptr<OneDGrid> Dune::GridFactory<Dune::OneDGrid>::
 createGrid()
 {
   // Prevent a crash when this method is called twice in a row
@@ -178,7 +178,7 @@ createGrid()
   // Hand over the grid and delete the member pointer
   auto tmp = grid_;
   grid_ = nullptr;
-  return tmp;
+  return std::unique_ptr<OneDGrid>(tmp);
 }
 
 void Dune::GridFactory<Dune::OneDGrid >::
