@@ -6,25 +6,25 @@
 
 #include <unistd.h>
 #include <iostream>
+#include <iomanip>
 #include <memory>
 #include <vector>
+#include <bitset>
 
 #include <dune/common/parallel/mpihelper.hh>
 #include <dune/common/float_cmp.hh>
 #include <dune/geometry/referenceelements.hh>
 #include <dune/grid/common/gridenums.hh>
 #include <dune/grid/common/mcmgmapper.hh>
-#include <dune/grid/io/file/vtk/vtkwriter.hh>
 #include <dune/grid/uggrid.hh>
 #include <dune/grid/utility/structuredgridfactory.hh>
 
 using namespace Dune;
 
 // A DataHandle class to exchange entries of a vector.
-template<class MapperT, int commCodim>
+template<class MapperT>
 class DataExchange
-  : public Dune::CommDataHandleIF<DataExchange<MapperT,
-            commCodim>,
+  : public Dune::CommDataHandleIF<DataExchange<MapperT>,
         double>
 {
 public:
@@ -34,9 +34,15 @@ public:
 
   //! constructor
   DataExchange(const MapperT &mapper,
+               std::bitset<MapperT::GridView::dimension+1> communicationCodims,
+               std::vector<std::size_t>& gatherCounter,
+               std::vector<std::size_t>& scatterCounter,
                UserDataType &userDataSend,
                UserDataType &userDataReceive)
     : mapper_(mapper),
+      communicationCodims_(communicationCodims),
+      gatherCounter_(gatherCounter),
+      scatterCounter_(scatterCounter),
       userDataSend_(userDataSend),
       userDataReceive_(userDataReceive)
   {}
@@ -45,7 +51,7 @@ public:
   //! returns true if data for this codim should be communicated
   bool contains (int dim, int codim) const
   {
-    return (codim == commCodim);
+    return communicationCodims_[codim];
   }
 
   //! returns true if size per entity of given dim and codim is a constant
@@ -69,9 +75,10 @@ public:
   void gather(MessageBuffer& buff, const EntityType& e) const
   {
     DataType x = mapper_.index(e);
+    gatherCounter_[mapper_.index(e)]++;
 
     userDataSend_[mapper_.index(e)][0] = x;
-    std::cout << "Process "
+    dverb << "Process "
               << Dune::MPIHelper::getCollectiveCommunication().rank()+1
               << " sends for entity "
               << mapper_.index(e)
@@ -93,7 +100,8 @@ public:
     buff.read(x);
 
     userDataReceive_[mapper_.index(e)][0] = x;
-    std::cout << "Process "
+    scatterCounter_[mapper_.index(e)]++;
+    dverb << "Process "
               << Dune::MPIHelper::getCollectiveCommunication().rank()+1
               << " received for entity "
               << mapper_.index(e)
@@ -104,6 +112,9 @@ public:
 
 private:
   const MapperT &mapper_;
+  const std::bitset<MapperT::GridView::dimension+1> communicationCodims_;
+  std::vector<std::size_t> &gatherCounter_;
+  std::vector<std::size_t> &scatterCounter_;
   UserDataType &userDataSend_;
   UserDataType &userDataReceive_;
 };
@@ -143,7 +154,7 @@ void checkMappers(const GridView &gridView)
   // make sure the number of entities is the same for
   // gridView.size() and iterating through the grid
   int numEntities = 0;
-  for (DUNE_UNUSED const auto& entity : entities(gridView, Dune::Codim<codim>()))
+  for ([[maybe_unused]] const auto& entity : entities(gridView, Dune::Codim<codim>()))
     ++ numEntities;
   if (numEntities != gridView.size(codim)) {
     DUNE_THROW(InvalidStateException,
@@ -207,146 +218,141 @@ struct checkMappersWrapper<2, 1, GridView>
 { static void check(const GridView &gv) { } };
 
 
-template <class GridView, int commCodim>
-void testCommunication(const GridView &gridView, bool isLeaf, bool printVTK=false)
+template <class GridView>
+void testCommunication(const GridView &gridView,
+                       std::bitset<GridView::dimension+1> communicationCodims,
+                       InterfaceType communicationInterface,
+                       CommunicationDirection communicationDirection,
+                       const std::set<PartitionType>& sendingPartitions,
+                       const std::set<PartitionType>& receivingPartitions)
 {
-  std::cout << gridView.comm().rank() + 1
-            << ": Testing communication for codim " << commCodim << " entities\n";
-
-  typedef Dune::MultipleCodimMultipleGeomTypeMapper<GridView> MapperType;
-  MapperType mapper(gridView, mcmgLayout(Codim<commCodim>{}));
-
-  std::cout << gridView.comm().rank() + 1
-            << ": Index set has " << mapper.size() << " codim " << commCodim << " entities\n";
-
   const int dim = GridView::dimension;
 
-  // create the user data arrays
-  typedef std::vector<Dune::FieldVector<double, 1> > UserDataType;
-  UserDataType userDataSend(gridView.size(commCodim), 0.0);
-  UserDataType userDataReceive(gridView.size(commCodim), 0.0);
-  UserDataType entityIndex(gridView.size(commCodim), -1e10);
-  UserDataType partitionType(gridView.size(commCodim), -1e10);
+  dverb << gridView.comm().rank() + 1
+            << ": Testing communication for the following codimensions " << communicationCodims << std::endl;
 
-  // write the partition type of each entity into the corrosponding
+  typedef Dune::MultipleCodimMultipleGeomTypeMapper<GridView> MapperType;
+  auto mcmgLayout = [&communicationCodims](GeometryType gt, int dimgrid)
+    {
+      return communicationCodims[dim - gt.dim()];
+    };
+  MapperType mapper(gridView, mcmgLayout);
+
+  // create the user data arrays
+  using UserDataType = std::vector<Dune::FieldVector<double, 1> >;
+
+  UserDataType userDataSend(mapper.size(), 0.0);
+  UserDataType userDataReceive(mapper.size(), 0.0);
+  // For each entity, count how many times 'gather' and 'scatter' have been called
+  std::vector<std::size_t> gatherCounter(mapper.size(), 0);
+  std::vector<std::size_t> scatterCounter(mapper.size(), 0);
+
+  // write the partition type of each entity into the corresponding
   // result array
-  for (const auto& entity : entities(gridView, Dune::Codim<commCodim>())) {
-    entityIndex[mapper.index(entity)]   = mapper.index(entity);
-    partitionType[mapper.index(entity)] = entity.partitionType();
+  for (const auto& element : elements(gridView))
+  {
+    Hybrid::forEach(std::make_index_sequence< dim+1 >{},
+      [&](auto codim){
+      if (communicationCodims[codim])
+      {
+        auto numberOfSubEntities = element.subEntities(codim);
+        for (std::size_t k = 0; k < numberOfSubEntities; k++)
+        {
+          const auto entity(element.template subEntity<codim>(k));
+
+          if (entity.partitionType() == Dune::BorderEntity)
+          {
+            const auto geometry = element.geometry();
+
+            auto referenceElement = Dune::referenceElement<double, dim>(element.type());
+            const auto entityGlobal = geometry.global(referenceElement.position(k, codim));
+            dverb << gridView.comm().rank()+1 << ": border codim "
+                  << codim << " entity "
+                  << mapper.index(entity) << " (" << entityGlobal
+                  << ")" << std::endl;
+          }
+        }
+      }
+    });
   }
 
   // initialize data handle (marks the nodes where some data was
   // send or received)
-  typedef DataExchange<MapperType, commCodim> MyDataHandle;
-  MyDataHandle datahandle(mapper,
-                          userDataSend,
-                          userDataReceive);
+  DataExchange<MapperType> datahandle(mapper,
+                                      communicationCodims,
+                                      gatherCounter,
+                                      scatterCounter,
+                                      userDataSend,
+                                      userDataReceive);
 
   // communicate the entities at the interior border to all other
   // processes
-  gridView.communicate(datahandle, Dune::InteriorBorder_All_Interface, Dune::ForwardCommunication);
+  gridView.communicate(datahandle, communicationInterface, communicationDirection);
 
-  //////////////////////////////////////////////////////
-  // Write results to disk
-  //////////////////////////////////////////////////////
-  if (printVTK)
+  // write the partition type of each entity into the corresponding
+  // result array
+  for (const auto& element : elements(gridView))
   {
-    std::cout << "Writing data to disk\n";
-    Dune::VTKWriter<GridView> writer(gridView);
-    if (commCodim == 0) {
-      writer.addCellData(userDataSend, "Send");
-      writer.addCellData(userDataReceive, "Receive");
-      writer.addCellData(partitionType, "Partition Type");
-      writer.addCellData(entityIndex, "Entity Index");
-    }
-    else if (commCodim == dim) {
-      writer.addVertexData(userDataSend, "Send");
-      writer.addVertexData(userDataReceive, "Receive");
-      writer.addVertexData(partitionType, "Partition Type");
-      writer.addVertexData(entityIndex, "Entity Index");
-    }
-
-    char fileName[1024];
-    sprintf(fileName, "test-parallel-ug-dim=%d-commCodim=%d", dim, commCodim);
-    if (isLeaf)
-      strcat(fileName, "-leaf");
-    else
+    Hybrid::forEach(std::make_index_sequence< dim+1 >{},
+      [&](auto codim)
     {
-      char levelName[10];
-      sprintf(levelName, "-level=%d", (gridView.template begin<commCodim>())->level());
-      strcat(fileName, levelName);
-    }
-    writer.write(fileName, Dune::VTK::ascii);
-    std::cout << "Done writing data to disk\n";
-  }
-}
-
-//! edge and face communication
-template <class GridView, int commCodim>
-class EdgeAndFaceCommunication
-{
-public:
-  static void test(const GridView &gridView)
-  {
-    const int dim = GridView::dimension;
-
-    std::cout << gridView.comm().rank() + 1
-              << ": Testing communication for codim " << commCodim << " entities\n";
-
-    typedef Dune::MultipleCodimMultipleGeomTypeMapper<GridView> MapperType;
-    MapperType mapper(gridView, mcmgLayout(Codim<commCodim>{}));
-
-    std::cout << gridView.comm().rank()+1 << ": Index set has " << mapper.size() << " codim " << commCodim << " entities\n";
-
-    // create the user data arrays
-    typedef std::vector<Dune::FieldVector<double, 1> > UserDataType;
-    UserDataType userDataSend(gridView.size(commCodim), 0.0);
-    UserDataType userDataReceive(gridView.size(commCodim), 0.0);
-    UserDataType entityIndex(gridView.size(commCodim), -1e10);
-    UserDataType partitionType(gridView.size(commCodim), -1e10);
-
-    // write the partition type of each entity into the corresponding
-    // result array
-    for (const auto& element : elements(gridView)) {
-      int numberOfSubEntities = element.subEntities(commCodim);
-      for (int k = 0; k < numberOfSubEntities; k++)
+      // TODO: Also check whether no communication happens when no communication
+      // is requested.
+      if (communicationCodims[codim])
       {
-        typedef typename GridView::template Codim<0>::Entity Element;
-        typedef typename Element::template Codim<commCodim>::Entity Entity;
-        const Entity entity(element.template subEntity<commCodim>(k));
-        entityIndex[mapper.index(entity)]   = mapper.index(entity);
-        partitionType[mapper.index(entity)] = entity.partitionType();
-
-        if (entity.partitionType() == Dune::BorderEntity)
+        auto numberOfSubEntities = element.subEntities(codim);
+        for (std::size_t k = 0; k < numberOfSubEntities; k++)
         {
-          const auto geometry = element.geometry();
-          const auto gt = geometry.type();
+          const auto entity(element.template subEntity<codim>(k));
 
-          auto referenceElement = Dune::referenceElement<double, dim>(gt);
-          const auto entityGlobal = geometry.global(referenceElement.position(k, commCodim));
-          std::cout << gridView.comm().rank()+1 << ": border codim "
-                    << commCodim << " entity "
-                    << mapper.index(entity) << " (" << entityGlobal
-                    << ")" << std::endl;
+          auto partitionTypes = entity.impl().partitionTypes();
+
+          // Check whether 'gather' has been called the appropriate number of times
+          std::size_t expectedNumberOfGatherCalls = 0;
+
+          for (const auto& pType : partitionTypes)
+            if (pType.first != gridView.comm().rank()
+                && (sendingPartitions.find(entity.partitionType())!=sendingPartitions.end())
+                && (receivingPartitions.find(pType.second))!=receivingPartitions.end())
+              expectedNumberOfGatherCalls++;
+
+          if (gatherCounter[mapper.index(entity)] != expectedNumberOfGatherCalls)
+          {
+            std::cerr << gridView.comm().rank() << ": UGGrid did not call 'gather' "
+                      << expectedNumberOfGatherCalls << " times, but "
+                      << gatherCounter[mapper.index(entity)] << " times on an entity!" << std::endl;
+            std::cerr << gridView.comm().rank() << ": Problematic entity: codim = " << codim
+                      << ",  partitionType = " << entity.partitionType()
+                      << ",  center = " << entity.geometry().center()
+                      << std::endl;
+            std::abort();
+          }
+
+          // Check whether 'scatter' has been called the appropriate number of times
+          std::size_t expectedNumberOfScatterCalls = 0;
+
+          for (const auto& pType : partitionTypes)
+            if (pType.first != gridView.comm().rank()
+                && (receivingPartitions.find(entity.partitionType()) != receivingPartitions.end())
+                && (sendingPartitions.find(pType.second) != sendingPartitions.end()))
+              expectedNumberOfScatterCalls++;
+
+          if (scatterCounter[mapper.index(entity)] != expectedNumberOfScatterCalls)
+          {
+            std::cerr << gridView.comm().rank() << ": UGGrid did not call 'scatter' "
+                      << expectedNumberOfScatterCalls << " times, but "
+                      << scatterCounter[mapper.index(entity)] << " times on an entity!" << std::endl;
+            std::cerr << gridView.comm().rank() << ": Problematic entity: codim = " << codim
+                      << ",  partitionType = " << entity.partitionType()
+                      << ",  center = " << entity.geometry().center()
+                      << std::endl;
+            std::abort();
+          }
         }
       }
-    }
-
-    // initialize data handle (marks the nodes where some data was
-    // send or received)
-    typedef DataExchange<MapperType, commCodim> MyDataHandle;
-    MyDataHandle datahandle(mapper,
-                            userDataSend,
-                            userDataReceive);
-
-    // communicate the entities at the interior border to all other
-    // processes
-    gridView.communicate(datahandle, Dune::InteriorBorder_All_Interface, Dune::ForwardCommunication);
-
-    std::cout << gridView.comm().rank() + 1
-              << ": Finished testing communication for codim " << commCodim << " entities\n";
+    });
   }
-};
+}
 
 template<typename Grid>
 class LoadBalance
@@ -427,8 +433,6 @@ class LoadBalance
   template<int codim, int... codimensions, typename=void>
   static void fillVector(const GridView& gv, Data& data)
   {
-    std::cout << "Filling vector for codim " << codim << "\n";
-
     const auto& idSet = gv.grid().localIdSet();
 
     for (const auto& entity : entities(gv, Dune::Codim<codim>(),
@@ -449,8 +453,6 @@ class LoadBalance
   template<int codim, int... codimensions, typename=void>
   static bool checkVector(const GridView& gv, const Data& data)
   {
-    std::cout << "Checking vector for codim " << codim << "\n";
-
     const auto& idSet = gv.grid().localIdSet();
 
     for (const auto& entity : entities(gv, Dune::Codim<codim>(),
@@ -496,14 +498,14 @@ public:
     // check for correctness
     checkVector<codimensions...>(gv, data);
 
-    std::cout << gv.comm().rank()
+    dverb << gv.comm().rank()
               << ": load balancing with data was successful." << std::endl;
   }
 };
 
 template<typename Grid>
 std::shared_ptr<Grid>
-setupGrid(bool simplexGrid, bool localRefinement, int refinementDim, bool refineUpperPart)
+setupGrid(bool simplexGrid, bool localRefinement, int refinementDim, bool refineUpperPart, int numCellsPerDim = 4)
 {
   const static int dim = Grid::dimension;
   StructuredGridFactory<Grid> structuredGridFactory;
@@ -511,7 +513,7 @@ setupGrid(bool simplexGrid, bool localRefinement, int refinementDim, bool refine
   Dune::FieldVector<double,dim> lowerLeft(0);
   Dune::FieldVector<double,dim> upperRight(1);
   std::array<unsigned int, dim> numElements;
-  std::fill(numElements.begin(), numElements.end(), 4);
+  std::fill(numElements.begin(), numElements.end(), numCellsPerDim);
   if (simplexGrid)
     return structuredGridFactory.createSimplexGrid(lowerLeft, upperRight, numElements);
   else
@@ -563,11 +565,11 @@ void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, b
   const LeafGV&  leafGridView = grid->leafGridView();
   const LevelGV& level0GridView = grid->levelGridView(0);
 
-  std::cout << "LevelGridView for level 0 has " << level0GridView.size(0)
+  dverb << "LevelGridView for level 0 has " << level0GridView.size(0)
             << " elements "  << level0GridView.size(dim - 1)
             << " edges and " << level0GridView.size(dim)
             << " nodes.\n";
-  std::cout << "LeafGridView has " << leafGridView.size(0)
+  dverb << "LeafGridView has " << leafGridView.size(0)
             << " elements, " << leafGridView.size(dim - 1)
             << " edges and " << leafGridView.size(dim)
             << " nodes.\n";
@@ -585,19 +587,44 @@ void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, b
   checkMappersWrapper<dim, 2, LeafGV>::check(leafGridView);
   checkMappersWrapper<dim, 3, LeafGV>::check(leafGridView);
 
-  // Test element and node communication on level view
-  testCommunication<LevelGV, 0>(level0GridView, false);
-  testCommunication<LevelGV, dim>(level0GridView, false);
-  EdgeAndFaceCommunication<LevelGV, dim-1>::test(level0GridView);
-  if (dim == 3)
-    EdgeAndFaceCommunication<LevelGV, 1>::test(level0GridView);
+  // Test communication
+  std::map<InterfaceType, std::set<PartitionType> > sendingPartitions;
+  sendingPartitions[InteriorBorder_InteriorBorder_Interface] = {InteriorEntity, BorderEntity};
+  sendingPartitions[InteriorBorder_All_Interface] = {InteriorEntity, BorderEntity};
+  sendingPartitions[Overlap_OverlapFront_Interface] = {OverlapEntity};
+  sendingPartitions[Overlap_All_Interface] = {OverlapEntity};
+  sendingPartitions[All_All_Interface] = {InteriorEntity, BorderEntity, OverlapEntity, FrontEntity, GhostEntity};
 
-  // Test element and node communication on leaf view
-  testCommunication<LeafGV, 0>(leafGridView, true);
-  testCommunication<LeafGV, dim>(leafGridView, true);
-  EdgeAndFaceCommunication<LeafGV, dim-1>::test(leafGridView);
-  if (dim == 3)
-    EdgeAndFaceCommunication<LeafGV, 1>::test(leafGridView);
+  std::map<InterfaceType, std::set<PartitionType> > receivingPartitions;
+  receivingPartitions[InteriorBorder_InteriorBorder_Interface] = {InteriorEntity, BorderEntity};
+  receivingPartitions[InteriorBorder_All_Interface] = {InteriorEntity, BorderEntity, OverlapEntity, FrontEntity, GhostEntity};
+  receivingPartitions[Overlap_OverlapFront_Interface] = {OverlapEntity, FrontEntity};
+  receivingPartitions[Overlap_All_Interface] = {InteriorEntity, BorderEntity, OverlapEntity, FrontEntity, GhostEntity};
+  receivingPartitions[All_All_Interface] = {InteriorEntity, BorderEntity, OverlapEntity, FrontEntity, GhostEntity};
+
+  // Test all communication interfaces
+  for (auto&& communicationInterface : {InteriorBorder_InteriorBorder_Interface,
+                                        InteriorBorder_All_Interface,
+                                        Overlap_OverlapFront_Interface,
+                                        Overlap_All_Interface,
+                                        All_All_Interface})
+  {
+    // The variable codimSet encodes a set of codimensions as a bitset.
+    // We loop over all possible sets.
+    for (std::size_t codimSet=0; codimSet<(1<<(dim+1)); codimSet++)
+    {
+      // TODO: Test BackwardCommunication, too!
+      testCommunication<LevelGV>(level0GridView, std::bitset<dim+1>(codimSet),
+                                 communicationInterface, ForwardCommunication,
+                                 sendingPartitions[communicationInterface],
+                                 receivingPartitions[communicationInterface]);
+
+      testCommunication<LeafGV>(leafGridView, std::bitset<dim+1>(codimSet),
+                                communicationInterface, ForwardCommunication,
+                                sendingPartitions[communicationInterface],
+                                receivingPartitions[communicationInterface]);
+    }
+  }
 
   ////////////////////////////////////////////////////
   //  Refine globally and test again
@@ -605,13 +632,10 @@ void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, b
 
   if (!localRefinement)
   {
-    std::cout << "Global refinement\n";
     grid->globalRefine(1);
   }
   else
   {
-    std::cout << "Local refinement\n";
-
     // mark all elements with x-coordinate < 0.5 for refinement
     for (const auto& element : elements(grid->leafGridView())) {
       int nRefine = 1;
@@ -630,12 +654,6 @@ void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, b
     grid->preAdapt();
     grid->adapt();
     grid->postAdapt();
-
-    // write the adapted grid to VTK
-    Dune::VTKWriter<LeafGV> writer(grid->leafGridView());
-    char fileName[1024];
-    sprintf(fileName, "adapted-grid-dim=%d", dim);
-    writer.write(fileName, Dune::VTK::ascii);
   }
 
   for (int i=0; i<=grid->maxLevel(); i++) {
@@ -652,20 +670,143 @@ void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, b
   checkMappersWrapper<dim, 2, LeafGV>::check(grid->leafGridView());
   checkMappersWrapper<dim, 3, LeafGV>::check(grid->leafGridView());
 
-  for (int i=0; i<=grid->maxLevel(); i++)
+  // Test all communication interfaces
+  for (auto&& communicationInterface : {InteriorBorder_InteriorBorder_Interface,
+                                        InteriorBorder_All_Interface,
+                                        Overlap_OverlapFront_Interface,
+                                        Overlap_All_Interface,
+                                        All_All_Interface})
   {
-    testCommunication<LevelGV, 0>(grid->levelGridView(i), false);
-    testCommunication<LevelGV, dim>(grid->levelGridView(i), false);
-    EdgeAndFaceCommunication<LevelGV, dim-1>::test(grid->levelGridView(i));
-    if (dim == 3)
-      EdgeAndFaceCommunication<LevelGV, 1>::test(grid->levelGridView(i));
-  }
-  testCommunication<LeafGV, 0>(grid->leafGridView(), true);
-  testCommunication<LeafGV, dim>(grid->leafGridView(), true);
-  EdgeAndFaceCommunication<LeafGV, dim-1>::test(grid->leafGridView());
-  if (dim == 3)
-    EdgeAndFaceCommunication<LeafGV, 1>::test(grid->leafGridView());
+    // The variable codimSet encodes a set of codimensions as a bitset.
+    // We loop over all possible sets.
+    for (std::size_t codimSet=0; codimSet<(1<<(dim+1)); codimSet++)
+    {
+      // TODO: Test BackwardCommunication, too!
+      for (int i=0; i<=grid->maxLevel(); i++)
+        testCommunication<LevelGV>(grid->levelGridView(i), std::bitset<dim+1>(codimSet),
+                                communicationInterface, Dune::ForwardCommunication,
+                                sendingPartitions[communicationInterface],
+                                receivingPartitions[communicationInterface]);
 
+      testCommunication<LeafGV>(leafGridView, std::bitset<dim+1>(codimSet),
+                                communicationInterface, Dune::ForwardCommunication,
+                                sendingPartitions[communicationInterface],
+                                receivingPartitions[communicationInterface]);
+    }
+  }
+}
+
+template<class GridView>
+bool isViewWithinBounds(const GridView& gridView,
+                        const Dune::FieldVector<double, GridView::dimensionworld>& lowerLeft,
+                        const Dune::FieldVector<double, GridView::dimensionworld>& upperRight)
+{
+  for (const auto& element : elements(gridView, Dune::Partitions::interior))
+  {
+    const auto center = element.geometry().center();
+    for (int dimIdx = 0; dimIdx < int(GridView::dimensionworld); ++dimIdx)
+      if (center[dimIdx] < lowerLeft[dimIdx] || center[dimIdx] > upperRight[dimIdx])
+        return false;
+  }
+  return true;
+}
+
+template<class GridView, std::enable_if_t<GridView::dimension==2, int> = 0>
+void checkDefaultBisectionPartition(const GridView& gridView, int numProcs, int rank)
+{
+  // check if all partitions exist (we don't care which processor has which partition)
+  int part = -1;
+  switch (numProcs){
+  case 1:
+    if (isViewWithinBounds(gridView, {0,0}, {1,1})) part = 0;
+    break;
+  case 2:
+    if (isViewWithinBounds(gridView, {0,0}, {0.5,1})) part = 0;
+    if (isViewWithinBounds(gridView, {0.5,0}, {1,1})) part = 1;
+    break;
+  case 3:
+    if (isViewWithinBounds(gridView, {0,0}, {1.0/3.0,1})) part = 0;
+    if (isViewWithinBounds(gridView, {1.0/3.0,0}, {1,0.5})) part = 1;
+    if (isViewWithinBounds(gridView, {1.0/3.0,0.5}, {1,1})) part = 2;
+    break;
+  case 4:
+    if (isViewWithinBounds(gridView, {0,0}, {0.5,0.5})) part = 0;
+    if (isViewWithinBounds(gridView, {0,0.5}, {0.5,1})) part = 1;
+    if (isViewWithinBounds(gridView, {0.5,0}, {1,0.5})) part = 2;
+    if (isViewWithinBounds(gridView, {0.5,0.5}, {1,1})) part = 3;
+    break;
+  case 8:
+    if (isViewWithinBounds(gridView, {0,0}, {0.25,0.5})) part = 0;
+    if (isViewWithinBounds(gridView, {0.25,0}, {0.5,0.5})) part = 1;
+    if (isViewWithinBounds(gridView, {0.5,0}, {0.75,0.5})) part = 2;
+    if (isViewWithinBounds(gridView, {0.75,0}, {1,0.5})) part = 3;
+    if (isViewWithinBounds(gridView, {0,0.5}, {0.25,1})) part = 4;
+    if (isViewWithinBounds(gridView, {0.25,0.5}, {0.5,1})) part = 5;
+    if (isViewWithinBounds(gridView, {0.5,0.5}, {0.75,1})) part = 6;
+    if (isViewWithinBounds(gridView, {0.75,0.5}, {1,1})) part = 7;
+    break;
+  default:
+    DUNE_THROW(Dune::NotImplemented, "Test for number of processors: " << numProcs);
+  }
+  const auto sumPart = gridView.comm().sum(part);
+  const auto n = numProcs-1;
+  if (sumPart != (n*n + n)/2)
+    DUNE_THROW(Dune::Exception, "Invalid partition!");
+}
+
+template<class GridView, std::enable_if_t<GridView::dimension==3, int> = 0>
+void checkDefaultBisectionPartition(const GridView& gridView, int numProcs, int rank)
+{
+  int part = -1;
+  switch (numProcs){
+  case 1:
+    if (isViewWithinBounds(gridView, {0,0,0}, {1,1,1})) part = 0;
+    break;
+  case 2:
+    if (isViewWithinBounds(gridView, {0,0,0}, {0.5,1,1})) part = 0;
+    if (isViewWithinBounds(gridView, {0.5,0,0}, {1,1,1})) part = 1;
+    break;
+  case 3:
+    if (isViewWithinBounds(gridView, {0,0,0}, {1.0/3.0,1,1})) part = 0;
+    if (isViewWithinBounds(gridView, {1.0/3.0,0,0}, {1,0.5,1})) part = 1;
+    if (isViewWithinBounds(gridView, {1.0/3.0,0.5,0}, {1,1,1})) part = 2;
+    break;
+  case 4:
+    if (isViewWithinBounds(gridView, {0,0,0}, {0.5,0.5,1})) part = 0;
+    if (isViewWithinBounds(gridView, {0,0.5,0}, {0.5,1,1})) part = 1;
+    if (isViewWithinBounds(gridView, {0.5,0,0}, {1,0.5,1})) part = 2;
+    if (isViewWithinBounds(gridView, {0.5,0.5,0}, {1,1,1})) part = 3;
+  case 8:
+    if (isViewWithinBounds(gridView, {0,0,0}, {0.5,0.5,0.5})) part = 0;
+    if (isViewWithinBounds(gridView, {0,0.5,0}, {0.5,1,0.5})) part = 1;
+    if (isViewWithinBounds(gridView, {0.5,0,0}, {1,0.5,0.5})) part = 2;
+    if (isViewWithinBounds(gridView, {0.5,0.5,0}, {1,1,0.5})) part = 3;
+    if (isViewWithinBounds(gridView, {0,0,0.5}, {0.5,0.5,1})) part = 4;
+    if (isViewWithinBounds(gridView, {0,0.5,0.5}, {0.5,1,1})) part = 5;
+    if (isViewWithinBounds(gridView, {0.5,0,0.5}, {1,0.5,1})) part = 6;
+    if (isViewWithinBounds(gridView, {0.5,0.5,0.5}, {1,1,1})) part = 7;
+  break;
+    default:
+    DUNE_THROW(Dune::NotImplemented, "Test for number of processors: " << numProcs);
+  return;
+  }
+  const auto sumPart = gridView.comm().sum(part);
+  const auto n = numProcs-1;
+  if (sumPart != (n*n + n)/2)
+    DUNE_THROW(Dune::Exception, "Invalid partition!");
+}
+
+template<int dim>
+void testDefaultLoadBalanceStructuredCube(const Dune::MPIHelper& mpiHelper)
+{
+  if (mpiHelper.rank() == 0)
+    std::cout << "Testing default load balancer for structured cube grid in " << dim << "D\n";
+
+  auto grid = setupGrid<UGGrid<dim>>(false, false, 0, false, mpiHelper.size()*2);
+  grid->loadBalance();
+  grid->globalRefine(1);
+  const auto& gridView = grid->leafGridView();
+  checkDefaultBisectionPartition(gridView, mpiHelper.size(), mpiHelper.rank());
 }
 
 int main (int argc , char **argv) try
@@ -715,6 +856,10 @@ int main (int argc , char **argv) try
     }
   }
 
+  // test default bisection load balancer for case with known partition
+  testDefaultLoadBalanceStructuredCube<2>(mpiHelper);
+  testDefaultLoadBalanceStructuredCube<3>(mpiHelper);
+
   return 0;
 }
 catch (Dune::Exception& e) {
