@@ -100,7 +100,7 @@ void makeHalfCircleQuad(Dune::UGGrid<2>& grid, bool boundarySegments, bool param
   // //////////////////////////////////////
 
   // The factory returns the grid object we have given it as a std::unique_ptr.
-  // Release it, the keep the std::unique_ptr destructor from killing it at
+  // Release it, to keep the std::unique_ptr destructor from killing it at
   // the end of this very method.
   // Yes, that is a bit weird.  But in particular the calling method tests whether
   // the GridFactory can reuse an existing UGGrid object, and I would like to keep
@@ -237,18 +237,13 @@ int main (int argc , char **argv) try
   //   Test whether I can create a grid with explict boundary segment ordering,
   //   but not parametrization functions (only 2d, so far)
   // ////////////////////////////////////////////////////////////////////////////
-#ifdef ModelP
-  {  // Force destruction of test grid right after the next test,
-     // because for parallel UG there can be only one (2d-)grid at once.
-#endif
+
   Dune::UGGrid<2> gridWithOrderedBoundarySegments;
   makeHalfCircleQuad(gridWithOrderedBoundarySegments, true, false);
 
   gridWithOrderedBoundarySegments.globalRefine(1);
   gridcheck(gridWithOrderedBoundarySegments);
-#ifdef ModelP
-  }
-#endif
+
   // ////////////////////////////////////////////////////////////////////////
   //   Check whether geometryInFather returns equal results with and
   //   without parametrized boundaries
@@ -256,7 +251,7 @@ int main (int argc , char **argv) try
 
   // Only the sequential UG can provide more than one 2d- or 3d-grid at once.
   // Therefore we do not perform the following test for parallel UGGrid.
-#if ! defined ModelP
+
   Dune::UGGrid<2> gridWithParametrization, gridWithoutParametrization;
 
   // make grids
@@ -288,7 +283,7 @@ int main (int argc , char **argv) try
     }
 
   }
-#endif
+
   // ////////////////////////////////////////////////////////////////////////
   //   Check whether copies of elements have the same global ID
   // ////////////////////////////////////////////////////////////////////////
