@@ -98,32 +98,6 @@ UGGrid < dim >::UGGrid(UGCollectiveCommunication comm)
   if (UG_NS<dim>::CreateBoundaryValueProblem(problemName.c_str(), 1,coeffs,1,upp) == nullptr)
     DUNE_THROW(GridError, "UG" << dim << "d::CreateBoundaryValueProblem() returned an error code!");
 
-  if (numOfUGGrids==0) {
-
-    if (dim==2)
-    {
-      char* nfarg = strdup("newformat DuneFormat2d");
-      if (UG_NS<dim>::CreateFormatCmd(1, &nfarg))
-        DUNE_THROW(GridError, "UG" << dim << "d::CreateFormat() returned an error code!");
-      free(nfarg);
-    }
-    if (dim==3)
-    {
-      char* newArgs[2];
-      for (int i=0; i<2; i++)
-        newArgs[i] = (char*)::malloc(50*sizeof(char));
-
-      sprintf(newArgs[0], "newformat DuneFormat3d" );
-      sprintf(newArgs[1], "V s1 : vt 1" ); // generates side vectors in 3D
-
-      if (UG_NS<dim>::CreateFormatCmd(2, newArgs))
-        DUNE_THROW(GridError, "UG" << dim << "d::CreateFormat() returned an error code!");
-
-      for (int i=0; i<2; i++)
-        free(newArgs[i]);
-    }
-  }
-
   numOfUGGrids++;
 
   dverb << "UGGrid<" << dim << "> with name " << name_ << " created!" << std::endl;
@@ -424,7 +398,7 @@ void UGGrid<dim>::getChildrenOfSubface(const typename Traits::template Codim<0>:
   childElementSides.resize(childElements.size());
 
   int i=0;
-  for (const auto f : list)
+  for (const auto &f : list)
   {
     // Set element
     typedef typename Traits::template Codim< 0 >::Entity Entity;
@@ -534,6 +508,108 @@ bool UGGrid < dim >::loadBalance(const std::vector<Rank>& targetProcessors, unsi
   return true;
 }
 
+#ifdef ModelP
+template <int dim>
+std::vector<typename UG_NS<dim>::DDD_IF> UGGrid<dim>::findDDDInterfaces(InterfaceType iftype,
+                                                                        int codim) const
+{
+  std::vector<typename UG_NS<dim>::DDD_IF> dddIfaces;
+
+  // UGGrid does not have overlap or front entities
+  if (iftype == Overlap_OverlapFront_Interface || iftype == Overlap_All_Interface)
+    return dddIfaces;
+
+  if (codim == 0)
+  {
+    switch (iftype) {
+    case InteriorBorder_InteriorBorder_Interface :
+      // do not communicate anything: Elements cannot be in
+      // the interior of two processes at the same time
+      break;
+    case InteriorBorder_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::ElementVHIF(multigrid_->dddContext()));
+      break;
+    case All_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::ElementSymmVHIF(multigrid_->dddContext()));
+      break;
+    default :
+      DUNE_THROW(GridError,
+                 "Element communication not supported for "
+                 "interfaces of type  "
+                 << iftype);
+    }
+  }
+  else if (codim == dim)
+  {
+    switch (iftype)
+    {
+    case InteriorBorder_InteriorBorder_Interface :
+      dddIfaces.push_back(UG_NS<dim>::BorderNodeSymmIF(multigrid_->dddContext()));
+      break;
+    case InteriorBorder_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::NodeInteriorBorderAllIF(multigrid_->dddContext()));
+      break;
+    case All_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::NodeAllIF(multigrid_->dddContext()));
+      break;
+    default :
+      DUNE_THROW(GridError,
+                 "Node communication not supported for "
+                 "interfaces of type  "
+                 << iftype);
+    }
+  }
+  else if (codim == dim-1)
+  {
+    switch (iftype)
+    {
+    case InteriorBorder_InteriorBorder_Interface :
+      dddIfaces.push_back(UG_NS<dim>::BorderEdgeSymmIF(multigrid_->dddContext()));
+      break;
+    case InteriorBorder_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::EdgeVHIF(multigrid_->dddContext()));
+      break;
+    case All_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::EdgeSymmVHIF(multigrid_->dddContext()));
+      break;
+    default :
+      DUNE_THROW(GridError,
+                 "Edge communication not supported for "
+                 "interfaces of type  "
+                 << iftype);
+    }
+  }
+  else if (codim == 1)
+  {
+    switch (iftype)
+    {
+    case InteriorBorder_InteriorBorder_Interface :
+      dddIfaces.push_back(UG_NS<dim>::BorderVectorSymmIF(multigrid_->dddContext()));
+      break;
+    case InteriorBorder_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::FacetInteriorBorderAllIF(multigrid_->dddContext()));
+      break;
+    case All_All_Interface :
+      dddIfaces.push_back(UG_NS<dim>::FacetAllAllIF(multigrid_->dddContext()));
+      break;
+    default :
+      DUNE_THROW(GridError,
+                 "Face communication not supported for "
+                 "interfaces of type  "
+                 << iftype);
+    }
+  }
+  else
+  {
+    DUNE_THROW(GridError,
+               "Communication codimension must be between 0 and " << dim << "!");
+  }
+
+  return dddIfaces;
+};
+#endif // ModelP
+
+
 template < int dim >
 void UGGrid < dim >::setPosition(const typename Traits::template Codim<dim>::Entity& e,
                                  const FieldVector<double, dim>& pos)
